"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var t = _interopRequireWildcard(require("@babel/types"));

var _core = require("@dubstep/core");

var _shared = require("./shared.js");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

/*
Copyright (c) 2018-2019 Uber Technologies, Inc.

This source code is licensed under the MIT license found in the
LICENSE file in the root directory of this source tree.
*/
function findTypedStyledCalls(p) {
  const styledLocalImportName = (0, _shared.getStyledLocalImportName)(p);
  const paths = [];
  p.traverse({
    CallExpression(path) {
      if (path.node.callee.name === styledLocalImportName && path.node.typeArguments) {
        paths.push(path);
      }
    }

  });
  return paths;
}

function findStyledElementsWithTheme(p) {
  return findTypedStyledCalls(p).filter(path => {
    return path.node.typeArguments.params.length === 2 && path.node.arguments.length && path.node.arguments[0].type === 'StringLiteral';
  });
}

function findStyledComponentsWithTheme(p) {
  return findTypedStyledCalls(p).filter(path => {
    return path.node.typeArguments.params.length === 3 && path.node.typeArguments.params[0].type === 'TypeofTypeAnnotation' && path.node.arguments.length && path.node.arguments[0].type === 'Identifier';
  });
}

class ThemeCache {
  // eslint-disable-next-line flowtype/no-weak-types
  constructor() {
    this.count = 0;
    this.visitedGenericTypes = {};
  }

  insert(node) {
    if (node.type === 'GenericTypeAnnotation') {
      this.count++;
      this.visitedGenericTypes[node.id.name] = {
        node,
        position: this.count
      };
    } else {
      this.count++;
    }
  }

  read(node) {
    if (node.type === 'GenericTypeAnnotation') {
      const value = this.visitedGenericTypes[node.id.name];
      return value ? value.node : null;
    } else {
      return null;
    }
  }

  position(node) {
    if (node.type === 'GenericTypeAnnotation' && this.visitedGenericTypes[node.id.name]) {
      return this.visitedGenericTypes[node.id.name].position;
    } else {
      return this.count;
    }
  }

}

function buildThemedStyledXName(position) {
  if (position <= 1) {
    return 'themedStyled';
  }

  return `themedStyled${position}`;
}

function visitStyledCall(path, themes) {
  const theme = path.node.typeArguments.params.pop();

  if (themes.read(theme)) {
    path.node.callee.name = buildThemedStyledXName(themes.position(theme));
  } else {
    themes.insert(theme);
    const nextCalleeName = buildThemedStyledXName(themes.position(theme));
    path.node.callee.name = nextCalleeName;
    const createThemedStyled = t.variableDeclaration('const', [t.variableDeclarator(t.identifier(nextCalleeName), t.callExpression(t.identifier('createThemedStyled'), []))]);
    createThemedStyled.declarations[0].init.callee.typeAnnotation = t.typeParameterInstantiation([themes.read(theme) || theme]);
    const parent = path.findParent(n => n.isVariableDeclaration());
    parent.insertBefore(createThemedStyled);
  }
}

async function styledV8ToThemedStyled(options) {
  await (0, _core.withJsFiles)(`${options.dir}/**/*.js`, async p => {
    if ((0, _shared.containsFlowComment)(p)) {
      const themes = new ThemeCache();
      const styledElements = findStyledElementsWithTheme(p);
      styledElements.forEach(path => visitStyledCall(path, themes));
      const styledComponents = findStyledComponentsWithTheme(p);
      styledComponents.forEach(path => visitStyledCall(path, themes));

      if (styledElements.length || styledComponents.length) {
        p.traverse({
          ImportDeclaration(path) {
            let shouldImportCreateThemedStyled = true;

            if (path.node.source.value === 'baseui') {
              for (const specifier of path.node.specifiers) {
                if (specifier.imported.name === 'createThemedStyled') {
                  shouldImportCreateThemedStyled = false;
                  break;
                }
              }

              if (shouldImportCreateThemedStyled) {
                const createThemedStyledSpecifier = t.importSpecifier(t.identifier('createThemedStyled'), t.identifier('createThemedStyled'));
                path.node.specifiers.push(createThemedStyledSpecifier);
              }
            }
          }

        });
      }
    }
  });
}

var _default = styledV8ToThemedStyled;
exports.default = _default;